@@ -1,14 +1,24 @@
 import logging
 import os
-import warnings
+import warnings as pywarnings
 import typing
 from collections import defaultdict, namedtuple
 from typing import Any, Dict, List, Optional, Text, Tuple
 
 from rasa import telemetry
+from rasa.core.constants import (
+    CONFUSION_MATRIX_STORIES_FILE,
+    REPORT_STORIES_FILE,
+    FAILED_STORIES_FILE,
+    SUCCESSFUL_STORIES_FILE,
+    STORIES_WITH_WARNINGS_FILE,
+)
 from rasa.core.policies.policy import PolicyPrediction
 from rasa.nlu.test import EntityEvaluationResult, evaluate_entities
-from rasa.shared.core.constants import POLICIES_THAT_EXTRACT_ENTITIES
+from rasa.shared.core.constants import (
+    POLICIES_THAT_EXTRACT_ENTITIES,
+    ACTION_UNLIKELY_INTENT_NAME,
+)
 from rasa.shared.exceptions import RasaException
 from rasa.shared.nlu.training_data.message import Message
 import rasa.shared.utils.io
@@ -16,13 +26,12 @@ from rasa.core.channels import UserMessage
 from rasa.shared.core.training_data.story_writer.yaml_story_writer import (
     YAMLStoryWriter,
 )
+from rasa.shared.core.training_data.structures import StoryStep
 from rasa.shared.core.domain import Domain
 from rasa.nlu.constants import (
-    ENTITY_ATTRIBUTE_TEXT,
     RESPONSE_SELECTOR_DEFAULT_INTENT,
     RESPONSE_SELECTOR_RETRIEVAL_INTENTS,
     TOKENS_NAMES,
-    ENTITY_ATTRIBUTE_CONFIDENCE,
 )
 from rasa.shared.nlu.constants import (
     INTENT,
@@ -38,48 +47,21 @@ from rasa.shared.nlu.constants import (
     RESPONSE_SELECTOR,
     FULL_RETRIEVAL_INTENT_NAME_KEY,
     TEXT,
-    ENTITY_ATTRIBUTE_GROUP,
-    ENTITY_ATTRIBUTE_ROLE,
+    ENTITY_ATTRIBUTE_TEXT,
 )
 from rasa.constants import RESULTS_FILE, PERCENTAGE_KEY
-from rasa.shared.core.events import (
-    ActionExecuted,
-    EntitiesAdded,
-    UserUttered,
-)
+from rasa.shared.core.events import ActionExecuted, EntitiesAdded, UserUttered
 from rasa.shared.core.trackers import DialogueStateTracker
 from rasa.shared.nlu.training_data.formats.readerwriter import TrainingDataWriter
 from rasa.shared.importers.importer import TrainingDataImporter
 from rasa.shared.utils.io import DEFAULT_ENCODING
+from rasa.utils.tensorflow.constants import QUERY_INTENT_KEY, SEVERITY_KEY
 
 if typing.TYPE_CHECKING:
     from rasa.core.agent import Agent
     from rasa.core.processor import MessageProcessor
     from rasa.shared.core.generator import TrainingDataGenerator
-
-    from typing_extensions import TypedDict
-
-    EntityPrediction = TypedDict(
-        "EntityPrediction",
-        {
-            ENTITY_ATTRIBUTE_TEXT: Text,
-            ENTITY_ATTRIBUTE_START: Optional[float],
-            ENTITY_ATTRIBUTE_END: Optional[float],
-            ENTITY_ATTRIBUTE_VALUE: Text,
-            ENTITY_ATTRIBUTE_CONFIDENCE: float,
-            ENTITY_ATTRIBUTE_TYPE: Text,
-            ENTITY_ATTRIBUTE_GROUP: Optional[Text],
-            ENTITY_ATTRIBUTE_ROLE: Optional[Text],
-            "additional_info": Any,
-        },
-        total=False,
-    )
-
-CONFUSION_MATRIX_STORIES_FILE = "story_confusion_matrix.png"
-REPORT_STORIES_FILE = "story_report.json"
-FAILED_STORIES_FILE = "failed_test_stories.yml"
-SUCCESSFUL_STORIES_FILE = "successful_test_stories.yml"
-
+    from rasa.shared.core.events import EntityPrediction
 
 logger = logging.getLogger(__name__)
 
@@ -89,6 +71,7 @@ StoryEvaluation = namedtuple(
         "evaluation_store",
         "failed_stories",
         "successful_stories",
+        "stories_with_warnings",
         "action_list",
         "in_training_data_fraction",
     ],
@@ -101,6 +84,88 @@ class WrongPredictionException(RasaException, ValueError):
     """Raised if a wrong prediction is encountered."""
 
 
+class WarningPredictedAction(ActionExecuted):
+    """The model predicted the correct action with warning."""
+
+    type_name = "warning_predicted"
+
+    def __init__(
+        self,
+        action_name_prediction: Text,
+        action_name: Optional[Text] = None,
+        policy: Optional[Text] = None,
+        confidence: Optional[float] = None,
+        timestamp: Optional[float] = None,
+        metadata: Optional[Dict] = None,
+    ):
+        """Creates event `action_unlikely_intent` predicted as warning.
+
+        See the docstring of the parent class for more information.
+        """
+        self.action_name_prediction = action_name_prediction
+        super().__init__(action_name, policy, confidence, timestamp, metadata)
+
+    def inline_comment(self) -> Text:
+        """A comment attached to this event. Used during dumping."""
+        return f"predicted: {self.action_name_prediction}"
+
+
+class WronglyPredictedAction(ActionExecuted):
+    """The model predicted the wrong action.
+
+    Mostly used to mark wrong predictions and be able to
+    dump them as stories.
+    """
+
+    type_name = "wrong_action"
+
+    def __init__(
+        self,
+        action_name_target: Text,
+        action_text_target: Text,
+        action_name_prediction: Text,
+        policy: Optional[Text] = None,
+        confidence: Optional[float] = None,
+        timestamp: Optional[float] = None,
+        metadata: Optional[Dict] = None,
+        predicted_action_unlikely_intent: bool = False,
+    ) -> None:
+        """Creates event for a successful event execution.
+
+        See the docstring of the parent class `ActionExecuted` for more information.
+        """
+        self.action_name_prediction = action_name_prediction
+        self.predicted_action_unlikely_intent = predicted_action_unlikely_intent
+        super().__init__(
+            action_name_target,
+            policy,
+            confidence,
+            timestamp,
+            metadata,
+            action_text=action_text_target,
+        )
+
+    def inline_comment(self) -> Text:
+        """A comment attached to this event. Used during dumping."""
+        comment = f"predicted: {self.action_name_prediction}"
+        if self.predicted_action_unlikely_intent:
+            return f"{comment} after {ACTION_UNLIKELY_INTENT_NAME}"
+        return comment
+
+    def as_story_string(self) -> Text:
+        """Returns the story equivalent representation."""
+        return f"{self.action_name}   <!-- {self.inline_comment()} -->"
+
+    def __repr__(self) -> Text:
+        """Returns event as string for debugging."""
+        return (
+            f"WronglyPredictedAction(action_target: {self.action_name}, "
+            f"action_prediction: {self.action_name_prediction}, "
+            f"policy: {self.policy}, confidence: {self.confidence}, "
+            f"metadata: {self.metadata})"
+        )
+
+
 class EvaluationStore:
     """Class storing action, intent and entity predictions and targets."""
 
@@ -200,11 +265,9 @@ class EvaluationStore:
     def serialise(self) -> Tuple[PredictionList, PredictionList]:
         """Turn targets and predictions to lists of equal size for sklearn."""
         texts = sorted(
-            list(
-                set(
-                    [e.get("text", "") for e in self.entity_targets]
-                    + [e.get("text", "") for e in self.entity_predictions]
-                )
+            set(
+                [str(e.get("text", "")) for e in self.entity_targets]
+                + [str(e.get("text", "")) for e in self.entity_predictions]
             )
         )
 
@@ -265,53 +328,15 @@ class EvaluationStore:
         return targets, predictions
 
 
-class WronglyPredictedAction(ActionExecuted):
-    """The model predicted the wrong action.
-
-    Mostly used to mark wrong predictions and be able to
-    dump them as stories."""
-
-    type_name = "wrong_action"
-
-    def __init__(
-        self,
-        action_name_target: Text,
-        action_text_target: Text,
-        action_name_prediction: Text,
-        policy: Optional[Text] = None,
-        confidence: Optional[float] = None,
-        timestamp: Optional[float] = None,
-        metadata: Optional[Dict] = None,
-    ) -> None:
-        """Creates event for a successful event execution.
-
-        See the docstring of the parent class `ActionExecuted` for more information.
-        """
-        self.action_name_prediction = action_name_prediction
-        super().__init__(
-            action_name_target,
-            policy,
-            confidence,
-            timestamp,
-            metadata,
-            action_text=action_text_target,
-        )
-
-    def inline_comment(self) -> Text:
-        """A comment attached to this event. Used during dumping."""
-        return f"predicted: {self.action_name_prediction}"
-
-    def as_story_string(self) -> Text:
-        return f"{self.action_name}   <!-- {self.inline_comment()} -->"
-
-
 class EndToEndUserUtterance(UserUttered):
     """End-to-end user utterance.
 
     Mostly used to print the full end-to-end user message in the
-    `failed_test_stories.yml` output file."""
+    `failed_test_stories.yml` output file.
+    """
 
     def as_story_string(self, e2e: bool = True) -> Text:
+        """Returns the story equivalent representation."""
         return super().as_story_string(e2e=True)
 
 
@@ -573,6 +598,39 @@ def _get_e2e_entity_evaluation_result(
                 )
 
 
+def _run_action_prediction(
+    processor: "MessageProcessor",
+    partial_tracker: DialogueStateTracker,
+    expected_action: Text,
+) -> Tuple[Text, PolicyPrediction, EntityEvaluationResult]:
+    action, prediction = processor.predict_next_action(partial_tracker)
+    predicted_action = action.name()
+
+    policy_entity_result = _get_e2e_entity_evaluation_result(
+        processor, partial_tracker, prediction
+    )
+
+    if (
+        prediction.policy_name
+        and predicted_action != expected_action
+        and _form_might_have_been_rejected(
+            processor.domain, partial_tracker, predicted_action
+        )
+    ):
+        # Wrong action was predicted,
+        # but it might be Ok if form action is rejected.
+        emulate_loop_rejection(partial_tracker)
+        # try again
+        action, prediction = processor.predict_next_action(partial_tracker)
+
+        # Even if the prediction is also wrong, we don't have to undo the emulation
+        # of the action rejection as we know that the user explicitly specified
+        # that something else than the form was supposed to run.
+        predicted_action = action.name()
+
+    return predicted_action, prediction, policy_entity_result
+
+
 def _collect_action_executed_predictions(
     processor: "MessageProcessor",
     partial_tracker: DialogueStateTracker,
@@ -584,57 +642,61 @@ def _collect_action_executed_predictions(
 
     action_executed_eval_store = EvaluationStore()
 
-    gold_action_name = event.action_name
-    gold_action_text = event.action_text
-    gold = gold_action_name or gold_action_text
+    expected_action_name = event.action_name
+    expected_action_text = event.action_text
+    expected_action = expected_action_name or expected_action_text
 
     policy_entity_result = None
+    prev_action_unlikely_intent = False
 
     if circuit_breaker_tripped:
         prediction = PolicyPrediction([], policy_name=None)
-        predicted = "circuit breaker tripped"
+        predicted_action = "circuit breaker tripped"
     else:
-        action, prediction = processor.predict_next_action(partial_tracker)
-        predicted = action.name()
-
-        policy_entity_result = _get_e2e_entity_evaluation_result(
-            processor, partial_tracker, prediction
+        predicted_action, prediction, policy_entity_result = _run_action_prediction(
+            processor, partial_tracker, expected_action
         )
 
-        if (
-            prediction.policy_name
-            and predicted != gold
-            and _form_might_have_been_rejected(
-                processor.domain, partial_tracker, predicted
+    predicted_action_unlikely_intent = predicted_action == ACTION_UNLIKELY_INTENT_NAME
+    if predicted_action_unlikely_intent and predicted_action != expected_action:
+        partial_tracker.update(
+            WronglyPredictedAction(
+                predicted_action,
+                expected_action_text,
+                predicted_action,
+                prediction.policy_name,
+                prediction.max_confidence,
+                event.timestamp,
+                metadata=prediction.action_metadata,
             )
-        ):
-            # Wrong action was predicted,
-            # but it might be Ok if form action is rejected.
-            emulate_loop_rejection(partial_tracker)
-            # try again
-            action, prediction = processor.predict_next_action(partial_tracker)
-
-            # Even if the prediction is also wrong, we don't have to undo the emulation
-            # of the action rejection as we know that the user explicitly specified
-            # that something else than the form was supposed to run.
-            predicted = action.name()
+        )
+        prev_action_unlikely_intent = True
+        predicted_action, prediction, policy_entity_result = _run_action_prediction(
+            processor, partial_tracker, expected_action
+        )
 
     action_executed_eval_store.add_to_store(
-        action_predictions=[predicted], action_targets=[gold]
+        action_predictions=[predicted_action], action_targets=[expected_action]
     )
 
     if action_executed_eval_store.has_prediction_target_mismatch():
         partial_tracker.update(
             WronglyPredictedAction(
-                gold_action_name,
-                gold_action_text,
-                predicted,
+                expected_action_name,
+                expected_action_text,
+                predicted_action,
                 prediction.policy_name,
                 prediction.max_confidence,
                 event.timestamp,
+                metadata=prediction.action_metadata,
+                predicted_action_unlikely_intent=prev_action_unlikely_intent,
             )
         )
-        if fail_on_prediction_errors:
+        if (
+            fail_on_prediction_errors
+            and predicted_action != ACTION_UNLIKELY_INTENT_NAME
+            and predicted_action != expected_action
+        ):
             story_dump = YAMLStoryWriter().dumps(partial_tracker.as_story().story_steps)
             error_msg = (
                 f"Model predicted a wrong action. Failed Story: " f"\n\n{story_dump}"
@@ -648,13 +710,25 @@ def _collect_action_executed_predictions(
                     "training stories and retrain."
                 )
             raise WrongPredictionException(error_msg)
-    else:
+    elif prev_action_unlikely_intent:
         partial_tracker.update(
-            ActionExecuted(
-                predicted,
+            WarningPredictedAction(
+                ACTION_UNLIKELY_INTENT_NAME,
+                predicted_action,
                 prediction.policy_name,
                 prediction.max_confidence,
                 event.timestamp,
+                prediction.action_metadata,
+            )
+        )
+    else:
+        partial_tracker.update(
+            ActionExecuted(
+                predicted_action,
+                prediction.policy_name,
+                prediction.max_confidence,
+                event.timestamp,
+                metadata=prediction.action_metadata,
             )
         )
 
@@ -719,19 +793,20 @@ async def _predict_tracker_actions(
             if entity_result:
                 policy_entity_results.append(entity_result)
 
-            tracker_eval_store.merge_store(action_executed_result)
-            tracker_actions.append(
-                {
-                    "action": action_executed_result.action_targets[0],
-                    "predicted": action_executed_result.action_predictions[0],
-                    "policy": prediction.policy_name,
-                    "confidence": prediction.max_confidence,
-                }
-            )
-            should_predict_another_action = processor.should_predict_another_action(
-                action_executed_result.action_predictions[0]
-            )
-            num_predicted_actions += 1
+            if action_executed_result.action_targets:
+                tracker_eval_store.merge_store(action_executed_result)
+                tracker_actions.append(
+                    {
+                        "action": action_executed_result.action_targets[0],
+                        "predicted": action_executed_result.action_predictions[0],
+                        "policy": prediction.policy_name,
+                        "confidence": prediction.max_confidence,
+                    }
+                )
+                should_predict_another_action = processor.should_predict_another_action(
+                    action_executed_result.action_predictions[0]
+                )
+                num_predicted_actions += 1
 
         elif use_e2e and isinstance(event, UserUttered):
             # This means that user utterance didn't have a user message, only intent,
@@ -772,6 +847,44 @@ def _in_training_data_fraction(action_list: List[Dict[Text, Any]]) -> float:
     return len(in_training_data) / len(action_list) if action_list else 0
 
 
+def _sort_trackers_with_severity_of_warning(
+    trackers_to_sort: List[DialogueStateTracker],
+) -> List[DialogueStateTracker]:
+    """Sort the given trackers according to 'severity' of `action_unlikely_intent`.
+
+    Severity is calculated by `IntentTEDPolicy` and is attached as
+    metadata to `ActionExecuted` event.
+
+    Args:
+        trackers_to_sort: Trackers to be sorted
+
+    Returns:
+        Sorted trackers in descending order of severity.
+    """
+    tracker_severity_scores = []
+    for tracker in trackers_to_sort:
+        max_severity = 0
+        for event in tracker.applied_events():
+            if (
+                isinstance(event, WronglyPredictedAction)
+                and event.action_name_prediction == ACTION_UNLIKELY_INTENT_NAME
+            ):
+                max_severity = max(
+                    max_severity,
+                    event.metadata.get(QUERY_INTENT_KEY, {}).get(SEVERITY_KEY, 0),
+                )
+        tracker_severity_scores.append(max_severity)
+
+    sorted_trackers_with_severity = sorted(
+        zip(tracker_severity_scores, trackers_to_sort),
+        # tuple unpacking is not supported in
+        # python 3.x that's why it might look a bit weird
+        key=lambda severity_tracker_tuple: -severity_tracker_tuple[0],
+    )
+
+    return [tracker for (_, tracker) in sorted_trackers_with_severity]
+
+
 async def _collect_story_predictions(
     completed_trackers: List["DialogueStateTracker"],
     agent: "Agent",
@@ -783,8 +896,9 @@ async def _collect_story_predictions(
     from tqdm import tqdm
 
     story_eval_store = EvaluationStore()
-    failed = []
-    success = []
+    failed_stories = []
+    successful_stories = []
+    stories_with_warnings = []
     correct_dialogues = []
     number_of_stories = len(completed_trackers)
 
@@ -811,11 +925,18 @@ async def _collect_story_predictions(
 
         if tracker_results.has_prediction_target_mismatch():
             # there is at least one wrong prediction
-            failed.append(predicted_tracker)
+            failed_stories.append(predicted_tracker)
             correct_dialogues.append(0)
         else:
+            successful_stories.append(predicted_tracker)
             correct_dialogues.append(1)
-            success.append(predicted_tracker)
+
+            if any(
+                isinstance(event, WronglyPredictedAction)
+                and event.action_name_prediction == ACTION_UNLIKELY_INTENT_NAME
+                for event in predicted_tracker.events
+            ):
+                stories_with_warnings.append(predicted_tracker)
 
     logger.info("Finished collecting predictions.")
 
@@ -835,8 +956,11 @@ async def _collect_story_predictions(
     return (
         StoryEvaluation(
             evaluation_store=story_eval_store,
-            failed_stories=failed,
-            successful_stories=success,
+            failed_stories=failed_stories,
+            successful_stories=successful_stories,
+            stories_with_warnings=_sort_trackers_with_severity_of_warning(
+                stories_with_warnings
+            ),
             action_list=action_list,
             in_training_data_fraction=in_training_data_fraction,
         ),
@@ -845,15 +969,36 @@ async def _collect_story_predictions(
     )
 
 
-def _log_stories(trackers: List[DialogueStateTracker], file_path: Text) -> None:
-    """Write given stories to the given file."""
+def _filter_step_events(step: StoryStep) -> StoryStep:
+    events = []
+    for event in step.events:
+        if (
+            isinstance(event, WronglyPredictedAction)
+            and event.action_name
+            == event.action_name_prediction
+            == ACTION_UNLIKELY_INTENT_NAME
+        ):
+            continue
+        events.append(event)
+    updated_step = step.create_copy(use_new_id=False)
+    updated_step.events = events
+    return updated_step
 
+
+def _log_stories(
+    trackers: List[DialogueStateTracker], file_path: Text, message_if_no_trackers: Text
+) -> None:
+    """Write given stories to the given file."""
     with open(file_path, "w", encoding=DEFAULT_ENCODING) as f:
         if not trackers:
-            f.write("# None of the test stories failed - all good!")
+            f.write(f"# {message_if_no_trackers}")
         else:
             stories = [tracker.as_story(include_source=True) for tracker in trackers]
-            steps = [step for story in stories for step in story.story_steps]
+            steps = [
+                _filter_step_events(step)
+                for story in stories
+                for step in story.story_steps
+            ]
             f.write(YAMLStoryWriter().dumps(steps))
 
 
@@ -867,6 +1012,7 @@ async def test(
     disable_plotting: bool = False,
     successes: bool = False,
     errors: bool = True,
+    warnings: bool = True,
 ) -> Dict[Text, Any]:
     """Run the evaluation of the stories, optionally plot the results.
 
@@ -882,6 +1028,7 @@ async def test(
         successes: boolean indicating whether to write down successful predictions or
             not
         errors: boolean indicating whether to write down incorrect predictions or not
+        warnings: boolean indicating whether to write down prediction warnings or not
 
     Returns:
         Evaluation summary.
@@ -897,10 +1044,10 @@ async def test(
 
     evaluation_store = story_evaluation.evaluation_store
 
-    with warnings.catch_warnings():
+    with pywarnings.catch_warnings():
         from sklearn.exceptions import UndefinedMetricWarning
 
-        warnings.simplefilter("ignore", UndefinedMetricWarning)
+        pywarnings.simplefilter("ignore", UndefinedMetricWarning)
 
         targets, predictions = evaluation_store.serialise()
 
@@ -912,12 +1059,14 @@ async def test(
             # Add conversation level accuracy to story report.
             num_failed = len(story_evaluation.failed_stories)
             num_correct = len(story_evaluation.successful_stories)
+            num_warnings = len(story_evaluation.stories_with_warnings)
             num_convs = num_failed + num_correct
-            if num_convs:
+            if num_convs and isinstance(report, Dict):
                 conv_accuracy = num_correct / num_convs
                 report["conversation_accuracy"] = {
                     "accuracy": conv_accuracy,
                     "correct": num_correct,
+                    "with_warnings": num_warnings,
                     "total": num_convs,
                 }
             report_filename = os.path.join(out_directory, REPORT_STORIES_FILE)
@@ -960,11 +1109,19 @@ async def test(
         _log_stories(
             story_evaluation.failed_stories,
             os.path.join(out_directory, FAILED_STORIES_FILE),
+            "None of the test stories failed - all good!",
         )
     if successes and out_directory:
         _log_stories(
             story_evaluation.successful_stories,
             os.path.join(out_directory, SUCCESSFUL_STORIES_FILE),
+            "None of the test stories succeeded :(",
+        )
+    if warnings and out_directory:
+        _log_stories(
+            story_evaluation.stories_with_warnings,
+            os.path.join(out_directory, STORIES_WITH_WARNINGS_FILE),
+            "No warnings for test stories",
         )
 
     return {
