@@ -258,6 +258,7 @@ class ResponseSelector(DIETClassifier):
         all_retrieval_intents: Optional[List[Text]] = None,
         responses: Optional[Dict[Text, List[Dict[Text, Any]]]] = None,
         finetune_mode: bool = False,
+        sparse_feature_sizes: Optional[Dict[Text, Dict[Text, List[int]]]] = None,
     ) -> None:
         """Declare instance variables with default values.
 
@@ -270,6 +271,7 @@ class ResponseSelector(DIETClassifier):
             responses: All responses defined in the data.
             finetune_mode: If `True` loads the model with pre-trained weights,
                 otherwise initializes it with random weights.
+            sparse_feature_sizes: Sizes of the sparse features the model was trained on.
         """
         component_config = component_config or {}
 
@@ -290,6 +292,7 @@ class ResponseSelector(DIETClassifier):
             entity_tag_specs,
             model,
             finetune_mode=finetune_mode,
+            sparse_feature_sizes=sparse_feature_sizes,
         )
 
     @property
@@ -931,7 +934,7 @@ class DIET2DIET(DIET):
 
         _, scores = self._tf_layers[
             f"loss.{LABEL}"
-        ].similarity_confidence_from_embeddings(
+        ].get_similarities_and_confidences_from_embeddings(
             sentence_vector_embed[:, tf.newaxis, :],
             self.all_labels_embed[tf.newaxis, :, :],
         )
