@@ -316,8 +316,10 @@ def replace_environment_variables() -> None:
         ]
         if not_expanded:
             raise RasaException(
-                f"Error when trying to expand the environment variables in '{value}'. "
-                f"Please make sure to also set these environment variables: '{not_expanded}'."
+                f"Error when trying to expand the "
+                f"environment variables in '{value}'. "
+                f"Please make sure to also set these "
+                f"environment variables: '{not_expanded}'."
             )
         return expanded_vars
 
