@@ -20,7 +20,8 @@ def require_response_keys(
 
         if response.get("text") is None and not response.get("custom"):
             return SchemaError(
-                "Missing 'text' or 'custom' key in response or null 'text' value in response.",
+                "Missing 'text' or 'custom' key in response or "
+                "null 'text' value in response.",
             )
 
     return True
