@@ -14,9 +14,10 @@ from typing import (
 )
 
 from rasa.shared.constants import DIAGNOSTIC_DATA
-from rasa.utils.tensorflow.model_data import RasaModelData, FeatureSignature
 from rasa.utils.tensorflow.constants import (
     LABEL,
+    IDS,
+    INTENT_CLASSIFICATION,
     SENTENCE,
     SEQUENCE_LENGTH,
     RANDOM_SEED,
@@ -35,6 +36,11 @@ from rasa.utils.tensorflow.constants import (
     CONSTRAIN_SIMILARITIES,
     MODEL_CONFIDENCE,
 )
+from rasa.utils.tensorflow.model_data import (
+    RasaModelData,
+    FeatureSignature,
+    FeatureArray,
+)
 import rasa.utils.train_utils
 from rasa.utils.tensorflow import layers
 from rasa.utils.tensorflow import rasa_layers
@@ -44,9 +50,14 @@ from rasa.utils.tensorflow.data_generator import (
     RasaBatchDataGenerator,
 )
 from tensorflow.python.keras.utils import tf_utils
+from rasa.shared.nlu.constants import TEXT
+from rasa.shared.exceptions import RasaException
 
 logger = logging.getLogger(__name__)
 
+LABEL_KEY = LABEL
+LABEL_SUB_KEY = IDS
+
 
 # noinspection PyMethodOverriding
 class RasaModel(TmpKerasModel):
@@ -270,13 +281,19 @@ class RasaModel(TmpKerasModel):
         return outputs
 
     def run_inference(
-        self, model_data: RasaModelData, batch_size: Union[int, List[int]] = 1
+        self,
+        model_data: RasaModelData,
+        batch_size: Union[int, List[int]] = 1,
+        output_keys_expected: Optional[List[Text]] = None,
     ) -> Dict[Text, Union[np.ndarray, Dict[Text, Any]]]:
         """Implements bulk inferencing through the model.
 
         Args:
             model_data: Input data to be fed to the model.
             batch_size: Size of batches that the generator should create.
+            output_keys_expected: Keys which are expected in the output.
+                The output should be filtered to have only these keys before
+                merging it with the output across all batches.
 
         Returns:
             Model outputs corresponding to the inputs fed.
@@ -292,7 +309,15 @@ class RasaModel(TmpKerasModel):
                 # We only need input, since output is always None and not
                 # consumed by our TF graphs.
                 batch_in = next(data_iterator)[0]
-                batch_out = self._rasa_predict(batch_in)
+                batch_out: Dict[
+                    Text, Union[np.ndarray, Dict[Text, Any]]
+                ] = self._rasa_predict(batch_in)
+                if output_keys_expected:
+                    batch_out = {
+                        key: output
+                        for key, output in batch_out.items()
+                        if key in output_keys_expected
+                    }
                 outputs = self._merge_batch_outputs(outputs, batch_out)
             except StopIteration:
                 # Generator ran out of batches, time to finish inferencing
@@ -528,7 +553,6 @@ class TransformerRasaModel(RasaModel):
         self.config = config
         self.data_signature = data_signature
         self.label_signature = label_data.get_signature()
-
         self._check_data()
 
         label_batch = RasaDataGenerator.prepare_batch(label_data.data)
@@ -539,6 +563,165 @@ class TransformerRasaModel(RasaModel):
         # set up tf layers
         self._tf_layers: Dict[Text, tf.keras.layers.Layer] = {}
 
+    def adjust_for_incremental_training(
+        self,
+        data_example: Dict[Text, Dict[Text, List[FeatureArray]]],
+        new_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+        old_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+    ) -> None:
+        """Adjusts the model for incremental training.
+
+        First we should check if any of the sparse feature sizes has decreased
+        and raise an exception if this happens.
+        If none of them have decreased and any of them has increased, then the
+        function updates `DenseForSparse` layers, compiles the model, fits a sample
+        data on it to activate adjusted layer(s) and updates the data signatures.
+
+        New and old sparse feature sizes could look like this:
+        {TEXT: {FEATURE_TYPE_SEQUENCE: [4, 24, 128], FEATURE_TYPE_SENTENCE: [4, 128]}}
+
+        Args:
+            data_example: a data example that is stored with the ML component.
+            new_sparse_feature_sizes: sizes of current sparse features.
+            old_sparse_feature_sizes: sizes of sparse features the model was
+                                      previously trained on.
+        """
+        self._check_if_sparse_feature_sizes_decreased(
+            new_sparse_feature_sizes=new_sparse_feature_sizes,
+            old_sparse_feature_sizes=old_sparse_feature_sizes,
+        )
+        if self._sparse_feature_sizes_have_increased(
+            new_sparse_feature_sizes=new_sparse_feature_sizes,
+            old_sparse_feature_sizes=old_sparse_feature_sizes,
+        ):
+            self._update_dense_for_sparse_layers(
+                new_sparse_feature_sizes, old_sparse_feature_sizes
+            )
+            self._compile_and_fit(data_example)
+
+    @staticmethod
+    def _check_if_sparse_feature_sizes_decreased(
+        new_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+        old_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+    ) -> None:
+        """Checks if the sizes of sparse features have decreased during fine-tuning.
+
+        Sparse feature sizes might decrease after changing the training data.
+        This can happen for example with `LexicalSyntacticFeaturizer`.
+        We don't support this behaviour and we raise an exception if this happens.
+
+        Args:
+            new_sparse_feature_sizes: sizes of current sparse features.
+            old_sparse_feature_sizes: sizes of sparse features the model was
+                                      previously trained on.
+
+        Raises:
+            RasaException: When any of the sparse feature sizes decrease
+                           from the last time training was run.
+        """
+        for attribute, new_feature_sizes in new_sparse_feature_sizes.items():
+            old_feature_sizes = old_sparse_feature_sizes[attribute]
+            for feature_type, new_sizes in new_feature_sizes.items():
+                old_sizes = old_feature_sizes[feature_type]
+                for new_size, old_size in zip(new_sizes, old_sizes):
+                    if new_size < old_size:
+                        raise RasaException(
+                            "Sparse feature sizes have decreased from the last time "
+                            "training was run. The training data was changed in a way "
+                            "that resulted in some features not being present in the "
+                            "data anymore. This can happen if you had "
+                            "`LexicalSyntacticFeaturizer` in your pipeline. "
+                            "The pipeline cannot support incremental training "
+                            "in this setting. We recommend you to retrain "
+                            "the model from scratch."
+                        )
+
+    @staticmethod
+    def _sparse_feature_sizes_have_increased(
+        new_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+        old_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+    ) -> bool:
+        """Checks if the sizes of sparse features have increased during fine-tuning.
+
+        If there's any sparse feature size that has increased after changing the
+        training data, we need to look for the corresponding `DenseForSparse` layer
+        and adjust it. On the other hand, if none of them have increased, we don't
+        need to change anything. This function helps us with making the decision.
+
+        Note that the function assumes that none of the sparse feature sizes
+        have decreased. In other words, it should get valid arguments in order
+        to function well.
+
+        Args:
+            new_sparse_feature_sizes: sizes of current sparse features.
+            old_sparse_feature_sizes: sizes of sparse features the model was
+                                      previously trained on.
+
+        Returns:
+            `True` if any of the sparse feature sizes has increased, `False` otherwise.
+        """
+        for attribute, new_feature_sizes in new_sparse_feature_sizes.items():
+            old_feature_sizes = old_sparse_feature_sizes[attribute]
+            for feature_type, new_sizes in new_feature_sizes.items():
+                old_sizes = old_feature_sizes[feature_type]
+                if sum(new_sizes) > sum(old_sizes):
+                    return True
+        return False
+
+    def _update_dense_for_sparse_layers(
+        self,
+        new_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+        old_sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]],
+    ) -> None:
+        """Updates `DenseForSparse` layers.
+
+        Updates sizes of `DenseForSparse` layers by comparing current sparse feature
+        sizes to old ones. This must be done before fine-tuning starts to account
+        for any change in the size of sparse features that might have happened
+        because of addition of new data.
+
+        Args:
+            new_sparse_feature_sizes: sizes of current sparse features.
+            old_sparse_feature_sizes: sizes of sparse features the model was
+                                      previously trained on.
+        """
+        for name, layer in self._tf_layers.items():
+            # `if` condition is necessary because only `RasaCustomLayer`
+            # can adjust sparse layers for incremental training by default.
+            if isinstance(layer, rasa_layers.RasaCustomLayer):
+                layer.adjust_sparse_layers_for_incremental_training(
+                    new_sparse_feature_sizes,
+                    old_sparse_feature_sizes,
+                    self.config[REGULARIZATION_CONSTANT],
+                )
+
+    def _compile_and_fit(
+        self, data_example: Dict[Text, Dict[Text, List[FeatureArray]]]
+    ) -> None:
+        """Compiles modified model and fits a sample data on it.
+
+        Args:
+            data_example: a data example that is stored with the ML component.
+        """
+        self.compile(optimizer=tf.keras.optimizers.Adam(self.config[LEARNING_RATE]))
+        label_key = LABEL_KEY if self.config[INTENT_CLASSIFICATION] else None
+        label_sub_key = LABEL_SUB_KEY if self.config[INTENT_CLASSIFICATION] else None
+
+        model_data = RasaModelData(
+            label_key=label_key, label_sub_key=label_sub_key, data=data_example
+        )
+        self._update_data_signatures(model_data)
+        data_generator = RasaBatchDataGenerator(model_data, batch_size=1)
+        self.fit(data_generator, verbose=False)
+
+    def _update_data_signatures(self, model_data: RasaModelData) -> None:
+        self.data_signature = model_data.get_signature()
+        self.predict_data_signature = {
+            feature_name: features
+            for feature_name, features in self.data_signature.items()
+            if TEXT in feature_name
+        }
+
     def _check_data(self) -> None:
         raise NotImplementedError
 
@@ -549,7 +732,6 @@ class TransformerRasaModel(RasaModel):
         """Prepares layers & loss for the final label prediction step."""
         self._prepare_embed_layers(predictor_attribute)
         self._prepare_embed_layers(LABEL)
-
         self._prepare_dot_product_loss(LABEL, self.config[SCALE_LOSS])
 
     def _prepare_embed_layers(self, name: Text, prefix: Text = "embed") -> None:
@@ -575,21 +757,30 @@ class TransformerRasaModel(RasaModel):
         )
 
     def _prepare_dot_product_loss(
-        self, name: Text, scale_loss: bool, prefix: Text = "loss"
+        self, name: Text, scale_loss: bool, prefix: Text = "loss",
     ) -> None:
-        self._tf_layers[f"{prefix}.{name}"] = layers.DotProductLoss(
+        self._tf_layers[f"{prefix}.{name}"] = self.dot_product_loss_layer(
             self.config[NUM_NEG],
-            self.config[LOSS_TYPE],
-            self.config[MAX_POS_SIM],
-            self.config[MAX_NEG_SIM],
-            self.config[USE_MAX_NEG_SIM],
-            self.config[NEGATIVE_MARGIN_SCALE],
-            scale_loss,
+            loss_type=self.config[LOSS_TYPE],
+            mu_pos=self.config[MAX_POS_SIM],
+            mu_neg=self.config[MAX_NEG_SIM],
+            use_max_sim_neg=self.config[USE_MAX_NEG_SIM],
+            neg_lambda=self.config[NEGATIVE_MARGIN_SCALE],
+            scale_loss=scale_loss,
             similarity_type=self.config[SIMILARITY_TYPE],
             constrain_similarities=self.config[CONSTRAIN_SIMILARITIES],
             model_confidence=self.config[MODEL_CONFIDENCE],
         )
 
+    @property
+    def dot_product_loss_layer(self) -> tf.keras.layers.Layer:
+        """Returns the dot-product loss layer to use.
+
+        Returns:
+            The loss layer that is used by `_prepare_dot_product_loss`.
+        """
+        return layers.SingleLabelDotProductLoss
+
     def _prepare_entity_recognition_layers(self) -> None:
         for tag_spec in self._entity_tag_specs:
             name = tag_spec.tag_name
